set mprint=no.

*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

define !LogReg (nrep = !charend('/')/
                iv_con = !default (NO) !charend('/') /
                iv_cat = !default (NO) !charend('/') /
                dep = !charend('/') /
                grp = !default(NO) !charend('/') /
                wgt =  !charend('/') /
                rwgt =  !charend('/') /
                cons =  !charend('/')/
                PSU =  !charend('/')/
                limit_criteria = !default(0) !charend('/')/
                infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!if (!upcase(!grp)=NO) !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
!let !j=!length(!concat(!blanks(!nGrp),"  ")).
!let !k=!length(!concat(!blanks(!nGrp)," ")).

/* List of all independent variables */
!let !iv=!null.
!if (!upcase(!iv_con)=NO) !then !let !iv=!iv_cat !ifend.
!if (!upcase(!iv_cat)=NO) !then !let !iv=!iv_con !ifend.
!if (!iv=!null) !then !let !iv=!concat(!iv_con," ",!iv_cat) !ifend.

/* List for contrast coding categorical variables */
!let !contrast=!null.
!if (!upcase(!iv_cat)=NO) !then !else
!do !i !in (!iv_cat)
!let !contrast=!concat(!contrast," /CONTRAST (",!i,")=Indicator").
!doend !ifend.


dataset close all.
get file !infile /keep=!if (!ngrp>0) !then !grp !ifend !PSU !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !dep !iv.
!if (!ngrp=0) !then compute !grp=1 !ifend.
missing values !grp ().
sort cases by !grp.       
compute MISS = (missing(!dep)).
!do !i !in (!iv).
if (missing(!i)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\sort.sav' /drop=MISS.


*** COMPUTE ESTIMATES ***.
     
get file 'c:\temp\sort.sav'.
split file by !grp.            
weight by !wgt.

oms /select tables /if subtype='Variables in the Equation' /destination format=sav outfile='c:\temp\coef.sav' /TAG="par".
oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2.sav' /TAG="R2".
LOGISTIC REGRESSION VARIABLES !dep
  /METHOD=ENTER !iv 
  !contrast
  /CRITERIA=PIN(.05) POUT(.10) ITERATE(20) CUT(.5).
omsend tag="par".
omsend tag="R2".

get file='C:\temp\coef.sav' /keep=var1 to !concat("var",!j)  B.
select if !concat("var",!k)="Step 1".
select if not missing(B).
exe.
delete var !concat("var",!k).
rename var (!concat("var",!j) B=IV STAT).

!if (!nGrp=0) !then !else
rename var (var1 to !concat("var",!nGrp) =!grp ).
!ifend.
alter type IV (a25).
save outfile='c:\temp\111.sav'.

get file='C:\temp\R2.sav' /keep=!if (!nGrp=0) !then !else var1 to !concat("var",!nGrp) !ifend CoxSnellRSquare NagelkerkeRSquare .
!if (!nGrp=0) !then !else
rename var (var1 to !concat("var",!nGrp) =!grp ).
!ifend.
VARSTOCASES
  /MAKE STAT FROM CoxSnellRSquare NagelkerkeRSquare
  /INDEX=IV(STAT) 
  !if (!ngrp>0) !then /KEEP=!grp !ifend
  /NULL=KEEP.
alter type IV (a25).
compute IV=concat("  ",IV).
add files file='c:\temp\111.sav' /file=*.
recode IV ("Constant"=" Constant").
!if (!ngrp=0) !then compute !grp=1 !ifend.
sort cases by !grp IV .
save outfile=!quote(!concat('c:\temp\',!wgt,'.sav')).


* Replicates.

!do !i=1 !to !nrep.
dataset close all.
get file 'c:\temp\sort.sav'.
split file by !grp.            
weight by !concat(!rwgt,!i).

oms /select tables /if subtype='Variables in the Equation' /destination format=sav outfile='c:\temp\coef.sav' /TAG="par".
oms /select tables /if subtype='Model Summary' /destination format=sav outfile='c:\temp\R2.sav' /TAG="R2".
LOGISTIC REGRESSION VARIABLES !dep
  /METHOD=ENTER !iv 
  !contrast
  /CRITERIA=PIN(.05) POUT(.10) ITERATE(20) CUT(.5).
omsend tag="par".
omsend tag="R2".

get file='C:\temp\coef.sav' /keep=var1 to !concat("var",!j)  B.
select if !concat("var",!k)="Step 1".
select if not missing(B).
exe.
delete var !concat("var",!k).
rename var (!concat("var",!j) B=IV STATR).
!if (!nGrp=0) !then !else
rename var (var1 to !concat("var",!nGrp) =!grp ).
!ifend.
alter type IV (a25).
save outfile='c:\temp\222.sav'.

get file='C:\temp\R2.sav' /keep=!if (!nGrp=0) !then !else var1 to !concat("var",!nGrp) !ifend CoxSnellRSquare NagelkerkeRSquare .
!if (!nGrp=0) !then !else
rename var (var1 to !concat("var",!nGrp) =!grp ).
!ifend.
VARSTOCASES
  /MAKE STATR FROM CoxSnellRSquare NagelkerkeRSquare
  /INDEX=IV(STATR) 
  !if (!ngrp>0) !then /KEEP=!grp !ifend
  /NULL=KEEP.
alter type IV (a25).
compute IV=concat("  ",IV).
add files file='c:\temp\222.sav' /file=*.
recode IV ("Constant"=" Constant").
!if (!ngrp=0) !then compute !grp=1 !ifend.
sort cases by !grp IV.
!if (!i=1) !then
save outfile=!quote(!concat('c:\temp\',!rwgt,'.sav')).
!else
add files file=!quote(!concat('c:\temp\',!rwgt,'.sav')) /file=*.
sort cases by !grp IV .
save outfile=!quote(!concat('c:\temp\',!rwgt,'.sav')).
!ifend !doend.


*** COMBINE RESULTS ***.

dataset close ALL.
match files table =!quote(!concat('c:\temp\',!wgt,'.sav')) /file=!quote(!concat('c:\temp\',!rwgt,'.sav'))
  /by !grp IV.
exe.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute var=(statr-STAT)**2.

aggregate outfile=* /break=!grp IV /STAT= mean(STAT) /var=sum(var).
compute var=!cons*var.


*** COMPUTE SE ***.

compute SE=sqrt(var).
formats STAT (f8.3)/ SE (f10.6).
save outfile='c:\temp\stats.sav'.


*** COMPUTE N ***.

get file='c:\temp\sort.sav'.
weight by !wgt.
aggregate /break=!grp /N_total=N.
filter by VALIDN.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /N_total=mean(N_total).
aggregate outfile=* /break=!grp /NU_psu=N /N_total=mean(N_total).
filter off.
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\stats.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
exe.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend.
!doend !ifend.

compute IV=Ltrim(IV).
exe.
omsend tag="crap".


*** OUTPUT TABLES ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !if (!nGrp>0) !then !grp !ifend IV STAT SE !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT=LIST NOCASENUM
  !if (!ngrp=0) !then /TITLE=!quote(!concat("Regression of ",!dep," on ",!iv)) 
  !else /TITLE=!quote(!concat("Logistic regression of ",!dep," on ",!iv," by ",!grp)) !ifend
  /CELLS=NONE.

delete variables !if (!ngrp=0) !then !grp !ifend  var.
var lab  
   NU_cases "No of unweighted cases included in analysis" 
   N_cases "No of weighted cases included in analysis"
   NU_psu "No of unweighted primary sampling units in the analysis"
   N_total "Total number of weighted students in data".

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !l !in (!limit_criteria)
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then
var lab FLAG_cases !quote(!concat("Flag if NU_cases < ",!l," ")).
!else
var lab FLAG_psu !quote(!concat("Flag if NU_psu < ",!l," ")).
!ifend !doend !ifend.

omsend tag="case".
restore.
!enddefine.

******************************************************************************************.
set mprint=yes.
!LogReg         nrep = 165/
                iv_con = civconf  ATINCULT/
                iv_cat = cob /
                dep = gender /
                grp = year/
                wgt =  FWGT /
                rwgt = RWGT /
                cons = 1/
                PSU =  Schoolid/
                limit_criteria = 30 5/
                infile = 'C:\WorkSync\NAP-CC\Analysis\AnalysisData\ReportingData_110722.sav'/.
set mprint=no.
******************************************************************************************.

