*** Created by Eveline Gebhardt, Australian Council for Educational Research ***.

set mprint=no.
DEFINE !UNIVAR_PV (nrep = !charend('/')/
           STAT = !charend('/')/
           dep = !charend('/')/
           grp = !default(NOGRP) !charend('/')/
           wgt = !charend('/')/
           rwgt = !charend('/')/
           cons = !charend('/')/
           PSU = !charend('/')/
           limit_criteria = !default(0) !charend('/')/
           infile = !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables TEXTS /destination viewer=no /Tag="NoViewer".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null.
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
!if (!ngrp=0) !then 
!let !x=2.
!else.
/* x is nGrp + 1 */
!let !x=!length(!concat(!blanks(!nGrp)," ")).
!ifend.
title !concat('(number of group variables is ',!nGrp,')(nGrp + 1 = ',!x,')').

dataset close all.
GET FILE !infile 
   /KEEP !if (!nGrp>0) !then !grp !ifend !psu !dep !wgt !do !i= 1 !to !nrep !concat(!rwgt,!i) !doend.
!if(!nGrp=0) !then compute !grp=1 !ifend.
alter type !grp (a10).
missing values !grp ().
SORT CASES BY !grp.
compute MISS=0.
!do !d !in (!dep).
if (missing(!d)) MISS=1.
!doend.
compute VALIDN=1-MISS.
formats VALIDN (f1.0).
save outfile='c:\temp\a.sav' /drop=MISS.


*** COMPUTE ESTIMATE ***.

oms /select tables /if subtypes='Report' /destination format=sav outfile='c:\temp\fullwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='FullWeight'.
WEIGHT BY !wgt.
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
omsend tag='FullWeight'.


* REPLICATES.   

oms /select tables /if subtypes='Report' /destination format=sav numbered="RepWeight" outfile='c:\temp\repwgt.sav' 
   /COLUMNS DIMNAMES = ["Variables"] /tag='RepWeight'.
!do !i= 1 !to !nrep.
WEIGHT BY !concat(!rwgt,!i).
means !dep !if (!nGrp>0) !then !do !g !in (!grp) !concat("by ",!g) !doend !else by !grp !ifend /cells=!STAT.
!doend.
omsend tag='RepWeight'.


*** COMBINE FILES ***.

!if (!ngrp=0) !then !let !ngrp=1 !ifend.

get file='c:\temp\repwgt.sav' /drop= Command_ Subtype_ Label_.
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).
rename vars (!dep=!do !d !in (!dep) !concat("R_",!d) !doend).
save outfile='c:\temp\repwgt2.sav'.

get file='c:\temp\fullwgt.sav' /drop= Command_ Subtype_ Label_.
sort cases by !do !n=1 !to !nGrp !concat("Var",!n) !doend !concat(" Var",!x).

match files table=* /file='c:\temp\repwgt2.sav' /by !do !n=1 !to !nGrp !concat(" Var",!n) !doend !concat(" Var",!x).
exe.

rename vars (!concat("Var",!x)=statistic).
!do !s !in (!STAT)
!if (!upcase(!s)=MEAN) !then recode statistic ("Mean"="MEAN") !ifend.
!if (!upcase(!s)=STDDEV) !then recode statistic("Std. Deviation"="STDDEV") !ifend.
!if (!upcase(!s)=MEDIAN) !then recode statistic("Median"="MEDIAN") !ifend.
!if (!upcase(!s)=SKEW) !then recode statistic("Skewness"="SKEW") !ifend.
!if (!upcase(!s)=VARIANCE) !then recode statistic("Variance"="VARIANCE") !ifend.
!if (!upcase(!s)=KURT) !then recode statistic("Kurtosis"="KURT") !ifend.
!doend.

!if (!nGrp > 0) !Then
rename vars (!do !n=1 !to !nGrp !concat("Var",!n) !doend =!grp).
!Ifend.

* Delete totals (Note: this might not work in all languages).
!do !g !in (!grp).
select if (substr(!g,1,3)<>"Tot").
!doend.
alter type !grp (a10).


*** COMPUTE SAMPLING VARIANCE (U) ***.

!do !d !in(!dep) 
COMPUTE !concat('var_',!d) = !cons*((!concat('r_',!d) - !d)**2).   
!doend.

AGGREGATE OUTFILE =*
   /PRESORTED /BREAK=!if (!nGrp > 0) !then !grp !ifend statistic 
   /!do !d !in(!dep) !d !doend=MEAN(!do !d !in(!dep) !d !doend) 
   /!do !d !in(!dep) !concat('var_',!d) !doend=SUM(!do !d !in(!dep) !concat('var_',!d) !doend) .

!let !k=!null.
!do !d !in (!dep)
!let !k=!concat(!k,",var_",!d).
!doend.
compute pv_var=mean(!substr(!k,2)).
exe.


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

!let !k=!null.
!do !d !in (!dep)
!let !k=!concat(!k,",",!d).
!doend.
compute STAT=mean(!substr(!k,2)).

do repeat a=!dep/b=pvar1 to pvar5.
compute b=(a-STAT)**2.
end repeat.

compute pvmerr=.25*(sum(pvar1 to pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute SE=sqrt(pv_var+1.2*pvmerr).
exe.
FORMATS STAT (F8.3) / SE (F10.6).

save outfile='c:\temp\z.sav'.


*** COMPUTE N ***.

get file='c:\temp\a.sav'.
filter by VALIDN.
weight by !wgt.
aggregate outfile='c:\temp\y.sav' /break=!grp /NU_cases=NU /N_cases=N.
weight off.
aggregate outfile=* /break=!grp !PSU /temp=N .
aggregate outfile=* /break=!grp /NU_psu=N .
filter off.
match files /file='c:\temp\y.sav' /file=* /by !grp.
match files file='c:\temp\z.sav' /TABLE=* /by !grp.
formats N_cases (f10.0).
exe.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then delete var !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then.
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend !doend !ifend.
exe.
omsend tag="NoViewer".


*** OUTPUT TABLES ***.

OMS /SELECT TABLES headings logs
   /IF COMMANDS = ["Summarize"] SUBTYPES = ["Case Processing Summary"]
   /DESTINATION   VIEWER = NO /TAG = "SumNoCaseProc".

!do !s !in (!STAT).
Temp.
Select if (statistic=!quote(!upcase(!s))).

SUMMARIZE
  /TABLES= !if (!upcase(!grp)="NOGRP") !then !else !grp !ifend statISTIC STAT SE 
   !if (!limit_criteria=0) !then N_cases NU_cases NU_psu !else FLAG_cases FLAG_psu  !ifend
  /FORMAT=LIST NOCASENUM !if (!nGrp > 0) !then TOTAL !ifend
  !if (!nGrp=0) !then /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!head(!dep), ' to ',!substr(!dep,!Index(!dep,PV5))))
  !else /TITLE=!quote(!concat(!upcase(!s),' and SE of ',!head(!dep), ' to ',!substr(!dep,!Index(!dep,PV5)),' by ',!grp)) !ifend
  /CELLS=NONE.
!doend 

OMSEND TAG=["SumNoCaseProc"].

var lab  
   NU_cases "No of unweighted cases included in analysis" 
   N_cases "No of weighted cases included in analysis"
   NU_psu "No of unweighted primary sampling units in the analysis".
!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !l !in (!limit_criteria)
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then
var lab FLAG_cases !quote(!concat("Flag if NU_cases < ",!l," ")).
!else
var lab FLAG_psu !quote(!concat("Flag if NU_psu < ",!l," ")).
!ifend !doend !ifend.

delete variables !if (!upcase(!grp)="NOGRP") !then !grp !ifend !dep !do !d !in (!dep) !concat("var_",!d) !doend
    pv_var pvar1 pvar2 pvar3 pvar4 pvar5 pvmerr.
restore.
set mprint=no.
!enddefine.


