set mprint=no.

*** Created by Eveline Gebhardt & Alexander Daraganov, Australian Council for Educational Research ***.

*** Performance statistics by national quarters of an index variable.
*** No random value is added to the index variable to make the 
*** variable continuous. 
*** The index is transformed into a group variable that indicate national
*** quartiles. Perfomance and index statistics are computed within each 
*** quartile.

define !PVWLEQRT_NoRV
                (pv = !charend('/') 
                /wle = !charend('/')
                /STAT = !charend('/')
                /infile = !charend('/')
                /cnt = !charend('/')
                /grp = !default(NOGRP) !charend('/') 
                /psu = !charend('/') 
                /wgt = !charend('/') 
                /rwgt = !charend('/')
                /nrep = !charend('/')
                /cons = !charend('/')
                /limit_criteria = !default(0) !charend('/')).

preserve.
SET Olang=English OVars names ONumbers values TVars names TNumbers values.
omsend.
oms /select headings tables texts/destination viewer=no /tag="crap".

/* nGrp is the number of group variables */
!if (!upcase(!grp)="NOGRP") !then
!let !nGrp=!null
!Else
!do !g !in (!grp)
!let !nGrp=!concat(!nGrp," ")
!doend !ifend.
!let !nGrp=!length(!nGrp).
title !ngrp.

get file=!infile /keep=!cnt !psu !if (!ngrp>0) !then !grp !ifend !wgt !concat(!rwgt,1) to !concat(!rwgt,!nrep) !PV !wle.
!if (!ngrp=0) !then compute !grp=1 !ifend.
sort cases by !cnt.
weight off.
select if not missing (!wle).
select if not missing (!grp).


* COMPUTE CUMULATIVE COUNT WITHIN COUNTRIES.
*----------------------------------------------.
autorecode !cnt /into cnt# /print.
sort cases by cnt# !wle.

do if ($casenum=1 or lag(cnt#) <> cnt#).
compute cumfreq=!wgt. 
else if (cnt#=lag(cnt#)).
compute cumfreq=!wgt + lag(cumfreq). 
end if.
sort cases by !cnt.
save outfile='C:\temp\temp.sav'.


* DEFINE CUTSCORES.
*----------------------------------------------.
weight by !wgt.
aggregate /presorted /break=!cnt  /total=max(cumfreq).
compute cut25=(total/100)*25.
compute cut50=(total/100)*50.
compute cut75=(total/100)*75.
exe.


* CREATE PERCENTILE GROUPS.
*--------------------------------------.
if (cumfreq<cut25) !concat("Q_",!wle)=1.
if (cumfreq>=cut25 & cumfreq<cut50) !concat("Q_",!wle)=2.
if (cumfreq>=cut50 & cumfreq<cut75) !concat("Q_",!wle)=3.
if (cumfreq>=cut75) !concat("Q_",!wle)=4.
formats !concat("Q_",!wle) (f1.0).
save outfile='C:\temp\quarters.sav'.


*** COMPUTE STATISTIC ***.

dataset close ALL.
get file='C:\temp\quarters.sav' /keep !cnt !grp !wgt !wle !PV !concat("Q_",!wle).
sort cases by !cnt !grp !concat("q_",!wle).
weight by !wgt.
aggregate outfile = !quote(!concat('C:\temp\',!wgt,'.sav')) /presorted /break=!cnt !grp !concat("q_",!wle)
   /PV_stat.1 to PV_stat.5=!STAT(!PV) /INDEX_stat=!STAT(!wle).  


*** REPLICATES ***.

!do !i= 1 !to !nrep.
dataset close all.
get file='C:\temp\quarters.sav' /keep !cnt !grp !concat(!rwgt,!i) !wle !PV !concat("Q_",!wle).
sort cases by !cnt !grp !concat("q_",!wle).
weight by !concat(!rwgt,!i).
aggregate outfile = !quote(!concat('C:\temp\',!rwgt,!i,'.sav')) /presorted /break=!cnt !grp !concat("q_",!wle)
   /PV_statR.1 to PV_statR.5=!STAT(!pv) /INDEX_statR=!STAT(!wle).  
!doend.    
    
*** COMBINE RESULTS ***.

dataset close ALL.
!let !nper=50.
!if ( !length(!substr(!blanks(!nrep),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
	add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !nrep /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
!else
	!let !modls=!nrep.
	!let !intgr=0.
	!Do !iter=!nrep !to !nper !by -!nper
		add files file= !quote(!concat('C:\temp\',!rwgt,!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !by -1 /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
		!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
		!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
		save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
	!doend
	!if (!modls>0) !then
		!if (!modls>1) !then
			add files file= !quote(!concat('C:\temp\',!rwgt,'1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,!e,'.sav')) !doend .
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!else
			get file =!quote(!concat('C:\temp\',!rwgt,'1.sav')).
			!let !intgr=!length(!Concat(!blanks(!intgr), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_temp_',!intgr,'.sav')).
		!ifend
	!ifend
	!if ( !length(!substr(!blanks(!intgr),!length(!concat(!blanks(!nper ), !blanks(1)))))=0 ) !then
		add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !intgr /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
	!else
		!let !modls=!intgr.
		!let !intgr1=0.
		!do !iter=!intgr !to !nper !by -!nper
			add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_',!iter,'.sav')) !do !e=!length(!substr(!blanks(!iter),2)) !to !length(!substr(!blanks(!iter),!length(!blanks(!nper)))) !ny -1 /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend.
			!let !modls=!length(!substr(!blanks(!modls),!length(!concat(!blanks(1),!blanks(!nper))))).
			!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
			save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
		!doend
		!if (!modls>0) !then
			!if (!modls>1) !then
				add files file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')) !do !e=2 !to !modls /file=!quote(!concat('C:\temp\',!rwgt,'_temp_',!e,'.sav')) !doend .
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!else
				get file= !quote(!concat('C:\temp\',!rwgt,'_temp_1.sav')).
				!let !intgr1=!length(!Concat(!blanks(!intgr1), !blanks(1))).
				save outfile=!quote(!concat('c:\temp\',!rwgt,'_tempover_',!intgr1,'.sav')).
			!ifend
		!ifend
		add files file= !quote(!concat('c:\temp\',!rwgt,'_tempover_1.sav')) !do !e=2 !to !intgr1 /file=!quote(!concat('C:\temp\',!rwgt,'_tempover_',!e,'.sav')) !doend .
	!ifend
!ifend
exe.

sort cases by !cnt !grp !concat("q_",!wle).
match files file=*/table= !quote(!concat('C:\temp\',!wgt,'.sav'))/by !cnt !grp !concat("q_",!wle).
exec.


*** COMPUTE SAMPLING VARIANCE (U) ***.

compute INDEX_var=(INDEX_statr-INDEX_stat)**2.


do repeat a=PV_statr.1 PV_statr.2 PV_statr.3 PV_statr.4 PV_statr.5/
	  b=PV_stat.1 PV_stat.2 PV_stat.3 PV_stat.4 PV_stat.5/
	  c=PV_var1 to PV_var5.
compute c=(a-b)**2.
end repeat.

aggregate outfile=*/ break=!cnt !grp !concat("q_",!wle)/ 
          PV_stat.1 to PV_stat.5= mean(PV_stat.1 PV_stat.2 PV_stat.3 PV_stat.4 PV_stat.5)/ 
          PV_var1 to PV_var5 = sum(PV_var1 to PV_var5)/
          INDEX_stat=mean(INDEX_stat)/
          INDEX_VAR=sum(INDEX_VAR).

do repeat a=PV_var1 to PV_var5 INDEX_VAR.
compute a=!cons*a.
end repeat.

compute PV_VAR=mean(PV_var1 to PV_var5).


*** CALCULATING MEASUREMENT VARIANCE (Bm) ***.

compute PV_stat=mean(PV_stat.1 to PV_stat.5).

do repeat a=PV_stat.1 to PV_stat.5/b=PV_pvar1 to PV_pvar5.
compute b=(a-PV_stat)**2.
end repeat.

compute pvmerr=.25*(sum(PV_pvar1 to PV_pvar5)).


*** COMPUTE SE [V = U + (1+1/M)Bm] ***.

compute PV_SE=sqrt(pv_var+1.2*pvmerr).
compute INDEX_SE=sqrt(INDEX_VAR).
exe.
formats PV_stat INDEX_stat (f8.3)/ PV_SE INDEX_SE (f10.6).
save outfile='c:\temp\stats.sav' /keep= !cnt !grp !concat('Q_',!wle) PV_stat PV_SE INDEX_stat INDEX_SE.
omsend tag="crap".


*** COMPUTE N ***.

get file='C:\temp\quarters.sav' /keep= !cnt !psu !grp !wgt !concat("Q_",!wle).
weight off.
aggregate /break=!cnt !grp !concat("Q_",!wle) /NU_CASES=N /N_CASES=sum(!wgt).
aggregate outfile=*/break=!cnt !grp !concat("Q_",!wle) !psu 
  /NU_CASES=mean(NU_CASES) /N_CASES=mean(N_CASES).
aggregate outfile=*/break=!cnt !grp !concat("Q_",!wle) 
  /NU_CASES=mean(NU_CASES) /N_CASES=mean(N_CASES) /NU_psu=N.

match files file='c:\temp\stats.sav' /file=* /by !cnt !grp !concat('Q_',!wle).
formats  NU_CASES N_CASES NU_psu (f10.0).
exe.


*** FLAGGING FOR OECD ***.

!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !b !in (!limit_criteria).
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then.
compute FLAG_cases=(NU_cases<!b).
formats FLAG_cases (f1.0).
!ifend.
!if (!length(!a)=2) !then
compute FLAG_psu=(NU_psu<!b).
formats FLAG_psu (f1.0).
!ifend !doend !ifend.


*** OUTPUT RESULTS ***.

oms /select tables /if subtypes='Case Processing Summary' /destination viewer=no /tag="case".
SUMMARIZE
  /TABLES= !cnt !if (!nGrp > 0) !then !grp !ifend !concat("Q_",!wle) INDEX_stat INDEX_SE PV_stat PV_SE
   !if (!limit_criteria=0) !then N_Cases NU_Cases NU_psu !else FLAG_cases FLAG_psu !ifend
  /FORMAT=LIST NOCASENUM
  !if (!nGrp > 0) !then /TITLE=!quote(!concat("Performance by national quartiles of ",!wle," and by ",!upcase(!grp),".")) 
  !else /TITLE=!quote(!concat("Performance by national quartiles of ",!wle,".")) !ifend
  /CELLS=NONE.

var lab  
   NU_cases "No of unweighted cases included in analysis" 
   N_cases "No of weighted cases included in analysis"
   NU_psu "No of unweighted primary sampling units in the analysis".
!if (!limit_criteria=0) !then !else
!let !a=!null.
!do !l !in (!limit_criteria)
!let !a=!concat(!a," ").
!if (!length(!a)=1) !then
var lab FLAG_cases !quote(!concat("Flag if NU_cases < ",!l," ")).
!else
var lab FLAG_psu !quote(!concat("Flag if NU_psu < ",!l," ")).
!ifend !doend !ifend.

omsend tag="case".
!if (!ngrp=0) !then delete var !grp !ifend.
restore.
!enddefine.


